/*
 * KawuiBawuiBo Game:
 *   - Copyright(c) 2000 Yongwoo's Park. All Rights Reserved.
 */

import java.io.*;
import java.net.*;
import java.util.Vector;
import java.util.Hashtable;
import java.util.Enumeration;
import java.util.StringTokenizer;

public class Lobby extends Thread {
   Hashtable chatList=null;   // chatting list
   Hashtable roomList=null;   // room list

   public Lobby() {
      chatList = new Hashtable();
      roomList = new Hashtable();
   }

   public void addPlayer(ServerPlayer player) {
      if(player.playerId != null && chatList.get(player.playerId) == null){
         int i=0;
         String message=null;

         if(chatList.size() == 0) {
            System.out.println("Lobby thread is started...");
//            start();
         }
         chatList.put(player.playerId, player);

         message = "300|UNICAST|"+player.playerId+"|"+player.playerId+"|["+player.playerId+"]  ʽÿ...";
         player.sendMessage(message);

         message = "300|BROADCAST|"+player.playerId+"|*|["+player.playerId+"] Ͽϴ..";
         broadcastMessage(message);

         sendRoomList(player.playerId);
         broadcastUserList("");
      }
   }

   public void takePlayerFromRoom(ServerPlayer player) {
      chatList.put(player.playerId, player);
   }

   public void removePlayer(ServerPlayer player) {
      chatList.remove(player.playerId);
      broadcastUserList("");
   }

   public ServerPlayer getPlayer(String playerId) {
      ServerPlayer player=null;

      if((player = (ServerPlayer)chatList.get(playerId)) != null) {
         return(player);
      }
      Enumeration enum = roomList.elements();
      while(enum.hasMoreElements()) {
         Room room = (Room)enum.nextElement();
         if((player = room.getPlayer(playerId)) != null) {
            return(player);
         }
      }
      return(null);
   }

   public void sendMessage(String playerId, String message) {
      ServerPlayer player = (ServerPlayer)chatList.get(playerId);
      if(player != null) {
         player.sendMessage(message);
      }
   }

   public void broadcastMessage(String message) {
      if(message == null) {
         return;
      }
      ServerPlayer player=null;
      Enumeration enum = chatList.elements();
      while(enum.hasMoreElements()) {
         player = (ServerPlayer)enum.nextElement();
         if(player != null) {
            player.sendMessage(message);
         }
      }
   }

   public void broadcastRoomList() {
      Enumeration enum = roomList.elements();

      String message = "310|**SERVER**|,";
      while(enum.hasMoreElements()) {
         Room room = (Room)enum.nextElement();
         message += room.roomId + ",";
      }
      if(message.endsWith(",")) {
         message = message.substring(0, message.length()-1);
      }
      broadcastMessage(message);
   }

   public void sendRoomList(String playerId) {
      Enumeration enum = roomList.elements();

      String message = "310|"+playerId+"|,";
      while(enum.hasMoreElements()) {
         Room room = (Room)enum.nextElement();
         message += room.roomId + ",";
      }
      if(message.endsWith(",")) {
         message = message.substring(0, message.length()-1);
      }
      sendMessage(playerId, message);
   }

   public void sendUserList(String playerId, String roomId) {
      Enumeration enum=null;
      if("".equals(roomId)) {
         enum = chatList.elements();
      } else {
         Room room = (Room)roomList.get(roomId);
         if(room == null) {
            return;
         }
         enum = room.playerList.elements();
      }
      String message = "312|"+playerId+"|"+roomId+"|";
      while(enum.hasMoreElements()) {
         ServerPlayer player = (ServerPlayer)enum.nextElement();
         message += player.playerId + ",";
      }
      if(message.endsWith(",")) {
         message = message.substring(0, message.length()-1);
      }
      sendMessage(playerId, message);
   }

   public void broadcastUserList(String roomId) {
      Enumeration enum=null;
      if("".equals(roomId)) {
         enum = chatList.elements();
      } else {
         Room room = (Room)roomList.get(roomId);
         if(room == null) {
            return;
         }
         enum = room.playerList.elements();
      }
      String message = "312|**SERVER**|"+roomId+"|";
      while(enum.hasMoreElements()) {
         ServerPlayer player = (ServerPlayer)enum.nextElement();
         message += player.playerId + ",";
      }
      if(message.endsWith(",")) {
         message = message.substring(0, message.length()-1);
      }
      broadcastMessage(message);
   }

   public void makeRoom(String playerId, String roomId) {
      ServerPlayer player = (ServerPlayer)chatList.get(playerId);

      Room room = new Room(roomId, player);
      roomList.put(roomId, room);

      room.start();
      System.out.println("Room thread, "+room.roomId+" is started...");
   }

   public boolean enterRoom(String playerId, String roomId) {
      ServerPlayer player = (ServerPlayer)chatList.get(playerId);

      Room room = (Room)roomList.get(roomId);
      String message=null;
      if(room != null) {
         if(room.playerList.size() < 2) {
            message = "322|"+playerId+"|"+roomId+"|SUCCESS";
            player.sendMessage(message);

            removePlayer(player);
            room.addPlayer(player);
            broadcastUserList(room.roomId);

            return(true);
         } else {
            message = "322|"+playerId+"|"+roomId+"|FULL";
            player.sendMessage(message);

            // 뿡    ٴ  ޽
            message = "399|"+playerId+"|["+roomId+"]   ϴ.";
            player.sendMessage(message);
         }
      } else {
         message = "322|"+playerId+"|"+roomId+"|NOROOM";

         // ش   ʴ´ٴ  ޽
         message = "399|"+playerId+"|["+roomId+"]  ʽϴ.";
         sendMessage(playerId, message);
      }
      return(false);
   }

   public void removeRoom(String roomId) {
      Room room = (Room)roomList.get(roomId);
      if(room != null) {
         roomList.remove(roomId);
         room = null;
         broadcastRoomList();
      }
   }

   public void joinGame(ServerPlayer player) {
   }

   public void leaveGame(ServerPlayer player) {
      removePlayer(player);
   }

   public void run() {
      while(true) {
         // Read Data
         try {
            sleep(100);
         } catch(InterruptedException ie) {
         }
         Enumeration e = chatList.elements();
         while(e.hasMoreElements()) {
            ServerPlayer player = (ServerPlayer)e.nextElement();
            try {
               String receive = null;
               receive = player.receiveMessage();

               if(receive == null || "".equals(receive)) {
                  continue;
               }

               // MW Game Protocol
               StringTokenizer st = new StringTokenizer(receive, "|");
               String command = st.nextToken();
               if("100".equals(command)) {
                  // εĳ ä 100|BROADCAST|playerId|*|chatStr
                  String type = st.nextToken();
                  String playerId = st.nextToken();
                  String target = st.nextToken();
                  String chatStr = st.nextToken();

                  String message = "300|BROADCAST|"+playerId+"|*|["+player.playerId+"] "+chatStr;
                  broadcastMessage(message);
               } else if("102".equals(command)) {
                  // ׷ĳ ä   102|GROUPCAST|playerId|groupId|chatStr
                  String type = st.nextToken();
                  String playerId = st.nextToken();
                  String target = st.nextToken();
                  String chatStr = st.nextToken();

//                  String message = "302|GROUPCAST|"+playerId+"|"+target+"|["+player.playerId+"] "+chatStr;
//                  groupcastMessage(target, message);
               } else if("104".equals(command)) {
                  // ĳ ä   104|[UNICAST|PRIVATE]|playerId|toPlayerId|chatStr
                  String type = st.nextToken();
                  String playerId = st.nextToken();
                  String target = st.nextToken();
                  String chatStr = st.nextToken();

                  String message = "304|UNICAST|"+playerId+"|"+target+"|["+player.playerId+"] "+chatStr;
                  sendMessage(target, message);
               } else if("110".equals(command)) {
                  // 븮Ʈû      110|playerId
                  String playerId = st.nextToken();

                  sendRoomList(playerId);
               } else if("112".equals(command)) {
                  // ڸƮû  112|playerId|roomId
                  String playerId = st.nextToken();
                  String roomId = st.nextToken();

                  sendUserList(playerId, roomId);
               } else if("120".equals(command)) {
                  // 游          120|playerId|roomId
                  String playerId = st.nextToken();
                  String roomId = st.nextToken();

                  makeRoom(playerId, roomId);
                  broadcastRoomList();
               } else if("122".equals(command)) {
                  //         122|playerId|roomId
                  String playerId = st.nextToken();
                  String roomId = st.nextToken();

                  if(enterRoom(playerId, roomId) == true) {
                     broadcastUserList("");
                  }
               } else if("180".equals(command)) {
                  //           180|JOIN|playerId
                  String type = st.nextToken();
                  String playerId = st.nextToken();

                  ServerPlayer pp = getPlayer(playerId);
                  if(pp != null) {
                     joinGame(pp);
                  }
               } else if("182".equals(command)) {
                  //           182|LEAVE|playerId
                  String type = st.nextToken();
                  String playerId = st.nextToken();

                  ServerPlayer pp = getPlayer(playerId);
                  if(pp != null) {
                     leaveGame(pp);
                  }
               }
            } catch(SocketException ne) {
               removePlayer(player);
            } catch(IOException ne) {
               removePlayer(player);
            }
         }
      }
//      System.out.println("Lobby thread is finished...");
   }

   public void destroyPlayer(ServerPlayer player) {
      try {
         if(player.is != null) {
            player.is.close();
            player.is = null;
         }
      } catch (Exception e) {
      }
      try {
         if(player.os != null) {
            player.os.close();
            player.os = null;
         }
		 } catch (Exception e) {
      }
      try {
         if(player.socket != null) {
            player.socket.close();
            player.socket = null;
         }
      } catch(Exception e) {
      }
   }
}
