/*
 * KawuiBawuiBo Game:
 *   - Copyright(c) 2000 Yongwoo's Park. All Rights Reserved.
 */

import java.net.*;
import java.io.*;
import java.util.*;

class Room extends Thread {
   static int roomCount=0;
   final static int GROUP_NUMBER = 2;

   String roomId=null;
   ServerPlayer hostPlayer=null;

   Vector playerList=null;
   int putCount=0;

   public Room(String roomId, ServerPlayer player) {
      int digit=0;

      this.roomId = roomId;
      this.hostPlayer = player;

      playerList = new Vector();
      addPlayer(player);

      System.out.println("ROOM: "+roomId);
   }

   synchronized public void addPlayer(ServerPlayer player) {
      if(!playerList.contains(player)) {
         playerList.addElement(player);
         player.room = this;

         broadcastUserList();

         if(playerList.size() == 2) {
            startGame();
         }
      }
   }

   synchronized public void removePlayer(ServerPlayer player) {
      if(player == null) {
         return;
      }
      player.room = null;

      if(playerList.contains(player)) {
         playerList.removeElement(player);
         player.lobby.takePlayerFromRoom(player);
         if(playerList.size() == 0) {
            player.lobby.removeRoom(roomId);
         }

         broadcastUserList();
      }
   }

   synchronized public void removePlayer(String playerId) {
      ServerPlayer player = getPlayer(playerId);
      removePlayer(player);
   }

   synchronized public ServerPlayer getPlayer(String playerId) {
      if(playerId == null) {
         return(null);
      }
      Enumeration enum = playerList.elements();
      while(enum.hasMoreElements()) {
         ServerPlayer player = (ServerPlayer)enum.nextElement();
         if(player.playerId.equals(playerId)) {
            return(player);
         }
      }
      return(null);
   }

   public void sendMessage(String playerId, String message) {
      ServerPlayer player = getPlayer(playerId);
      if(player != null) {
         player.sendMessage(message);
      }
   }

   public void broadcastMessage(String message) {
      if(message == null) {
         return;
      }
      Enumeration enum = playerList.elements();
      while(enum.hasMoreElements()) {
         ServerPlayer player = (ServerPlayer)enum.nextElement();
         player.sendMessage(message);
      }
   }

   void sendUserList(String playerId) {
      ServerPlayer player = getPlayer(playerId);
      if(player != null) {
         sendUserList(player);
      }
   }

   void sendUserList(ServerPlayer player) {
      Enumeration enum = playerList.elements();
      String message = "312|**SERVER**|"+roomId+"|";
      while(enum.hasMoreElements()) {
         message += ((ServerPlayer)enum.nextElement()).playerId + ",";
      }
      if(message.endsWith(",")) {
         message = message.substring(0, message.length()-1);
      }
      player.sendMessage(message);
   }

   void broadcastUserList() {
      Enumeration enum = playerList.elements();
      String message = "312|**SERVER**|"+roomId+"|";
      while(enum.hasMoreElements()) {
         ServerPlayer player = (ServerPlayer)enum.nextElement();
         message += player.playerId + ",";
      }
      if(message.endsWith(",")) {
         message = message.substring(0, message.length()-1);
      }
      broadcastMessage(message);
   }

   synchronized public void clearPae() {
      Enumeration enum = playerList.elements();
      while(enum.hasMoreElements()) {
         ServerPlayer player = (ServerPlayer)enum.nextElement();
         player.putPae = null;
      }
      putCount = 0;
   }

   synchronized public void startGame() {
      String message=null;

      clearPae();

      message = "799|**SERVER**|  մϴ.";
      broadcastMessage(message);

      message = "720|START|*";
      broadcastMessage(message);

      sendPoint();
   }

   synchronized public void putPae(ServerPlayer player, String pae) {
      String message=null;

      if((player != null)&&(player.putPae == null)) {
         putCount++;
         player.putPae = pae;
         if(putCount == 2) {
            finishGame();
            try {
               Thread.sleep(5000);  //  5  
            } catch(InterruptedException e) {
            }
            calculatePoint();
            startGame();
         }
      }
   }

   synchronized public void finishGame() {
      String message=null;

      message = "799|**SERVER**|  ҽϴ.";
      broadcastMessage(message);

      ServerPlayer player0 = (ServerPlayer)playerList.elementAt(0);
      ServerPlayer player1 = (ServerPlayer)playerList.elementAt(1);

      message = "722|PUT|"+player0.playerId+","+player0.putPae+"|"+player1.playerId+","+player1.putPae;
      broadcastMessage(message);
   }

   synchronized public void calculatePoint() {
      String message=null;

      ServerPlayer player0 = (ServerPlayer)playerList.elementAt(0);
      ServerPlayer player1 = (ServerPlayer)playerList.elementAt(1);

      if("KAWUI".equals(player0.putPae)) {
         if("KAWUI".equals(player1.putPae)) {
         } else if("BAWUI".equals(player1.putPae)) {
            player0.point -= 10;
            player1.point += 10;
         } else if("BO".equals(player1.putPae)) {
            player0.point += 10;
            player1.point -= 10;
         }
      } else if("BAWUI".equals(player0.putPae)) {
         if("KAWUI".equals(player1.putPae)) {
            player0.point += 10;
            player1.point -= 10;
         } else if("BAWUI".equals(player1.putPae)) {
         } else if("BO".equals(player1.putPae)) {
            player0.point -= 10;
            player1.point += 10;
         }
      } else if("BO".equals(player0.putPae)) {
         if("KAWUI".equals(player1.putPae)) {
            player0.point -= 10;
            player1.point += 10;
         } else if("BAWUI".equals(player1.putPae)) {
            player0.point += 10;
            player1.point -= 10;
         } else if("BO".equals(player1.putPae)) {
         }
      }
   }

   synchronized public void sendPoint() {
      String message=null;

      ServerPlayer player0 = (ServerPlayer)playerList.elementAt(0);
      ServerPlayer player1 = (ServerPlayer)playerList.elementAt(1);

      message = "724|POINT|"+player0.playerId+","+player0.point+"|"+player1.playerId+","+player1.point;

      broadcastMessage(message);
   }

   synchronized public void joinGame(ServerPlayer player) {
      String message=null;

      message = "799|"+player.playerId+"|["+player.playerId+"]  Ͽϴ.";
   }

   public void run() {
      String receive=null, message=null;
      Enumeration enum=null;
      ServerPlayer player=null;

      while(true) {
         // Read Data
         try {
            Thread.sleep(50);
         } catch(InterruptedException ie) {
         }
         enum = playerList.elements();
         while(enum.hasMoreElements()) {
            player = (ServerPlayer)enum.nextElement();
            try {
               receive = player.receiveMessage();
               if(receive == null || "".equals(receive)) {
                  continue;
               }
               // MW Game Protocol
               // nnn|arg1|arg2|arg3|...
               StringTokenizer st = new StringTokenizer(receive, "|");
               String command = st.nextToken();
               if("500".equals(command)) {
                  // εĳ ä 500|BROADCAST|playerId|*|chatStr
                  String type = st.nextToken();
                  String playerId = st.nextToken();
                  String target = st.nextToken();
                  String chatStr = st.nextToken();

                  message = "700|BROADCAST|"+playerId+"|*|["+player.playerId+"] "+chatStr;
                  broadcastMessage(message);
               } else if("502".equals(command)) {
                  // ׷ĳ ä   502|GROUPCAST|playerId|groupId|chatStr
                  String type = st.nextToken();
                  String playerId = st.nextToken();
                  String target = st.nextToken();
                  String chatStr = st.nextToken();

//                  message = "702|GROUPCAST|"+playerId+"|"+target+"|["+player.playerId+"] "+chatStr;
//                  groupcastMessage(target, message);
               } else if("504".equals(command)) {
                  // ĳ ä   504|[UNICAST|PRIVATE]|playerId|toPlayerId|chatStr
                  String type = st.nextToken();
                  String playerId = st.nextToken();
                  String toPlayerId = st.nextToken();
                  String chatStr = st.nextToken();

                  message = "704|UNICAST|"+playerId+"|"+toPlayerId+"|["+player.playerId+"] "+chatStr;
                  sendMessage(toPlayerId, message);
               } else if("512".equals(command)) {
                  // ڸƮ       512|playerId|roomId
                  String playerId = st.nextToken();
                  String roomId = st.nextToken();

                  player = getPlayer(playerId);
                  if(player != null) {
                     sendUserList(player);
                  }
               } else if("520".equals(command)) {
                  // ӽ       520|START|playerId
                  String type = st.nextToken();
                  String playerId = st.nextToken();

//                  startGame();
               } else if("522".equals(command)) {
                  // Ӱ      522|PUT|playerId|[KAWUI|BAWUI|BO]
                  String type = st.nextToken();
                  String playerId = st.nextToken();
                  String pae = st.nextToken();

                  player = getPlayer(playerId);
                  if(player != null) {
                     putPae(player, pae);
                  }
               } else if("524".equals(command)) {
                  //         524|POINT|playerId|point1
                  String type = st.nextToken();
                  String playerId = st.nextToken();
                  String point = st.nextToken();

                  sendPoint();
               } else if("580".equals(command)) {
                  //            580|JOIN_ROOM|playerId
                  String type = st.nextToken();
                  String playerId = st.nextToken();

                  message = "780|JOIN_ROOM|"+playerId;
                  sendMessage(playerId, message);

                  broadcastUserList();
               } else if("582".equals(command)) {
                  //            582|EXIT_ROOM|playerId
                  String type = st.nextToken();
                  String playerId = st.nextToken();

                  removePlayer(playerId);

                  message = "782|EXIT_ROOM|"+playerId;
                  sendMessage(playerId, message);

                  broadcastUserList();
               }
            } catch(SocketException e) { //Stream doesn't exist.
               removePlayer(player);
            } catch(IOException e) { //Stream doesn't exist.
               removePlayer(player);
            } catch(Exception e) {
            }
         }
      }
   }
}
